

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           vissim.h                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:    This file contains the macros, structures, and classes
      needed to define an API for creating visualizations into a scenario.
			
   Written by:     Alex Harper
*********************************************************************** */


#if !defined __VISSIM_H
#define __VISSIM_H


#include <stdio.h>

#include "memman.h"


// ******************************************************************


#define MM_LINE		0
#define MM_LABEL	1
#define VMAXLISTS	2

#define LEVEL_UNCLASS		0
#define LEVEL_SECRET		1
#define LEVEL_TOP_SECRET	2

#ifndef EARTH_RADIUS
#define EARTH_RADIUS    (6371221.3)
#endif

#define FTtoMET  0.3048
#define METtoFT  3.280839895
#define NMItoFT  6076.000
#define NMItoMET 1851.9648
#define MET2NMI  0.0005399568035


#define INTERFACE_FLAG_NULL       0x0000
#define INTERFACE_FLAG_MINICREW   0x0001

// fx flags
#define FX_FLAG_NULL              0x00
#define FX_FLAG_TRAIL             0x01
#define FX_FLAG_DAMAGE            0x02
#define FX_FLAG_MISSILE_FLAME_JET 0x04
#define FX_FLAG_ROTORS            0x08
#define FX_FLAG_SFX_IDLE          0x10
#define FX_FLAG_SFX_MOVING        0x20

#define PLATFORM_FLAG_NULL          0x000000
#define PLATFORM_FLAG_FLY           0x000001
#define PLATFORM_FLAG_DEAD          0x000002
#define PLATFORM_FLAG_DEAD_RECOG    0x000004
#define PLATFORM_FLAG_CONTROL       0x000008
#define PLATFORM_FLAG_VISIBLE       0x000010
#define PLATFORM_FLAG_UPDATE        0x000020
#define PLATFORM_FLAG_POSTMORTIUM   0x000040
#define PLATFORM_FLAG_EXTERNAL      0x000080
#define PLATFORM_FLAG_FORCED_UPDATE 0x000100
#define PLATFORM_FLAG_MOVEMENT      0x000200

#define PERFORM_QUERY_ID              0x00000001
//#define PERFORM_QUERY_BOUNDING_SPHERE 0x00000002
#define PERFORM_PLATFORM_DEACTIVATE   0x00000004
#define PERFORM_PLATFORM_INIT_EMITTER 0x00000008
#define PERFORM_PLATFORM_EMITTER      0x00000010
#define PERFORM_UPDATE_SENSORS        0x00000020


#define IFACEDATA_SCENARIO_CENTER 0x0001
#define IFACEDATA_SCENARIO_BOUNDS 0x0002

#define RADAR_MODE_SEARCH     0
#define RADAR_MODE_AQUISITION 1
#define RADAR_MODE_TRACK      2
#define RADAR_MODE_GUIDANCE   3


typedef struct {
   vector2d latlon;     // radians
} iface_scenario_center_type;


typedef struct {
   double length, width; // meters
} iface_scenario_bounds_type;


/*
typedef struct {
   vector3d pos;
   double radius;
} perform_bounding_sphere_type;
*/

/* **************************************************************
  This class is the base class for any GUI' controller for the system
************************************************************** */
class gui_type {

   public:
      int doneflag;

      gui_type() { doneflag = 0; }
      virtual ~gui_type() {}
      
      virtual void query_win_dim(int *v) = 0;
      virtual void reset_win_dim(int *v) = 0;
      virtual void init(int argc, char **argv) = 0;
};


/* **************************************************************
  This structure holds the data to manage a SF/X
************************************************************** */
typedef struct {

   vector3d xyz;   // local offset
   vector3d hpr;   // local offset
   double scale;   // f/x scale
} fx_params;


/* **************************************************************
  This structure holds the data to manage a SF/X
************************************************************** */
typedef struct {

   fx_params state;
   int count;   // for blades - # of blades
} fx_rotor_params;


/* **************************************************************
************************************************************** */
class correlate : public dbl_llist {

   public:
      int keycode;  // semantic code

      string_type objname;			// object filename (.flt)
      fx_params trail;				// engine trail parameters
      fx_params damage;				// damage f/x parameters
      fx_rotor_params rotor[4];			// helo blade rotor effects
      int num_rotors;				// # of rotors on object
      int sfx_moving_id;			// moving sound filename
      int sfx_idle_id;				// idle sound filename
      int fx_flag;				// various f/x bit flags
                      
      correlate() { fx_flag = FX_FLAG_NULL; }
      correlate(FILE *infile, char *token);
      virtual ~correlate() {}

      correlate *find(int code);
};


/* **************************************************************
************************************************************** */
class symbol_lut_type : public dbl_llist_manager {

   public:
      virtual ~symbol_lut_type() {}

      dbl_llist *find(int code);
      dbl_llist *find(char *code);
      void dump(FILE *outfile);
      int read_data(FILE *infile, char *token);
};


/* **************************************************************
  This class manages the semantic code -> 3D graphics correlation table
************************************************************** */
class platform_manager : public dbl_llist_manager {

   public:
      virtual ~platform_manager() { }
   
      int read_data(char *filename);
};


/* **************************************************************
  This class manages a text item on screen
************************************************************** */
class labeltype : public dbl_llist {

   protected:
      string_type label;		// text for the label

   public:
      vector2d pos;			// screen position
   
      void set_label(char *string);
      char *get_label() { return label.string; }
};


/* **************************************************************
  This class manages a 2D line on screen
************************************************************** */
class linetype : public dbl_llist {

   public:
      int data;				// misc data
      vector3d pt[2];			// endpts of line
};


/* **************************************************************
   This class handles the more frequently allocated data structures
************************************************************** */
class vmem_manager : public memman {

   public:
      vmem_manager(int maxlists, int maxbuffers) : memman(maxlists, maxbuffers) {}
      void *pop(int item, int size=0);
};


/* **************************************************************
   This class represents a correlation between a name, 2D symbol id,
   3D symbol id, and color
************************************************************** */
class symbol_correlation_type : public dbl_llist {

   public:
      string_type sym_label;
      int sym_id;
      string_type sym_sim;
      unsigned int sym_3D;
      unsigned int sym_2D;
      vector4uc sym_2D_color;
      string_type sym_2D_colorname;
      int sym_sds;
      int burst_warhead;
      int burst_fuze;
      int burst_quantity;
      int burst_rate;

      symbol_correlation_type();
      virtual ~symbol_correlation_type() {}
};


/* **************************************************************
   pos[0] = east;
   pos[1] = north;
   pos[2] = alt;
   
   rpy[0] = roll    y (rad) counterclockwise
   rpy[1] = pitch   x (rad) counterclockwise
   rpy[2] = heading z (rad) counterclockwise
   rpy = (0,0,0) is north

  This class reprents a snapshot of a platform's state
************************************************************** */
class plat_stat_type {

   public:
      vector3d init_pos;
      vector3d init_vel;
      double elapsed_time;

      vector3d pos;			// position
      vector3d acc;			// acceleration
      vector3d vel;			// velocity
      vector3d rpy;			// orientation
      vector3d drpy;			// angular velocity
      
      double timer;             // last update from sim (sim clock)
      double machine_time;	// last update/dead recon (computer clock)
      int    dead_recogn_flag;
      
      plat_stat_type();
      void dead_recog(double *currtime, double *multiplier, double *newtime);
};


/* **************************************************************
************************************************************** */
class basic_radar_data_type : public dbl_llist {

   public:
      int    id;
      double frequency;
      double freq_range;
      double power;
      double prf;
      double pulse_width;
      double beam_azimuth;
      double beam_az_sweep;
      double beam_elevation;
      double beam_ele_sweep;
      int    mode;
      symbol_correlation_type *str;

      basic_radar_data_type() { str = NULL; }
      virtual ~basic_radar_data_type() {}
};


class basic_platform;
class iface;


/* **************************************************************

   pos[0] = east;
   pos[1] = north;
   pos[2] = alt;
   
   rpy[0] = roll    y (rad) clockwise
   rpy[1] = pitch   x (rad) clockwise
   rpy[2] = heading z (rad) clockwise

************************************************************** */
class platform_data {

   protected:
      basic_platform *parent;
      
   public:

      platform_data(basic_platform *x) { parent = x; }
      virtual ~platform_data() {}
      
      virtual void label(dbl_llist_manager *label_manager) = 0;
      virtual void update(double *machine_time) = 0;
      virtual int  perform(iface *isim, int task, void *data);
      virtual void render(iface *isim) = 0;
      virtual int  init(iface *isim, correlate *basis, symbol_correlation_type *str) = 0;
};


/* **************************************************************
   This class is the base class for any simulation entity in the
   simulation
************************************************************** */
class basic_platform : public dbl_llist {

   protected:
      virtual void update_data(double *machine_time) = 0;

   public:
      platform_data *vis_data;
      plat_stat_type plat_clone; // positional data
      int platform_flag;         // various bit flags
      int id;                    // platform id
      int type;                  // correlation code

      basic_platform()  { type = 0; platform_flag = PLATFORM_FLAG_DEAD_RECOG | PLATFORM_FLAG_VISIBLE; vis_data = NULL; }
      virtual ~basic_platform() { if (vis_data) delete vis_data; }

      void label(dbl_llist_manager *label_manager);
      void update(iface *isim, double *currtime, double *multiplier, double *machine_time, int pause);
      virtual int perform(iface *isim, int task, void *data);
      virtual void update_platform(plat_stat_type *pc) {}
};


/* **************************************************************
   This class is the base class for any simulation manager
************************************************************** */
class iface : public dbl_llist_manager {

   public:
      unsigned int interface_flag;
      platform_manager platman;
      void *data_loader;
             
      iface();
      virtual ~iface() { }

      basic_platform *get_platform(int platid);
      void register_platform_data_loaderCB(void *func);

      virtual void init();
      virtual void cleanup() {}
      virtual void label(dbl_llist_manager *label_manager);
      virtual void parse_input(int argc, char **argv);
      virtual void update(double *machine_time) {}
      virtual void mcp(plat_stat_type *pc);
      virtual int  get_data(int type, void *data) { return 0; }
      virtual int  set_data(int type, void *data) { return 0; }
};


// ******************************************************************

// conversions
void deg_to_dms(double d_lat, double d_lon, char *lat, char *lon, int format);
void dms_to_deg(char *lat, char *lon, double *d_lat, double *d_lon);
void xy_to_deg(double x, double y, double *lat, double *lon);
void deg_to_xy(double lat, double lon, double *x, double *y);
void xy_to_rad(double x, double y, double *lat, double *lon);
void rad_to_xy(double lat, double lon, double *x, double *y);

void init_earth(double center_lat, double center_lon, double earth_radius);

extern double CONVERT_ctrlat;
extern double CONVERT_ctrlon;
extern double CONVERT_EarthRadius;

// ******************************************************************


extern vmem_manager vmemman;

// parse.cpp
extern string_type terrain_name;

extern double camera_dist;		// meters
extern double camera_height;		// meters
extern double camera_azimuth;	// radians
extern double camera_angular_velocity; // radians

extern int local_game_fogflag[3];
extern double local_game_fog_start;
extern double local_game_fog_end;
extern vector4uc local_game_fog_color;
extern int openglflag;
extern double backplane;

extern string_type overlay_name;
extern int overlay_intensity;

extern string_type map_indexname;
extern vector4uc tile_color;
extern int controller;
extern vector3d controller_speed;

extern symbol_lut_type symbol_lut;

extern binary_tree colortree;
extern unsigned int security_level;

extern int scramnet_flag;

extern string_type DATANAME_SEMOBJ;
extern string_type DATANAME_SETUP;

int  parse_setup(char *filename);
int  write_setup(char *filename);
int  init_color_lut(char *filename);

void register_setup_CB(void *setupCB);
void register_setup_out_CB(void *setup_outCB);

#endif

